@@ -1,8 +1,9 @@
 package Module::Build::PPMMaker;
 
 use strict;
+use Config;
 use vars qw($VERSION);
-$VERSION = '0.34_06';
+$VERSION = '0.35_01';
 $VERSION = eval $VERSION;
 
 # This code is mostly borrowed from ExtUtils::MM_Unix 6.10_03, with a
@@ -113,7 +114,9 @@ EOF
   my $ppd_file = "$dist{name}.ppd";
   my $fh = IO::File->new(">$ppd_file")
     or die "Cannot write to $ppd_file: $!";
-  $fh->binmode(":utf8") if $fh->can("binmode");
+  
+  $fh->binmode(":utf8") 
+    if $fh->can('binmode') && $] >= 5.008 && $Config{useperlio};
   print $fh $ppd;
   close $fh;
 
