@@ -3,6 +3,7 @@ package MBTest;
 use strict;
 
 use File::Spec;
+use File::Temp ();
 use File::Path ();
 
 
@@ -53,9 +54,10 @@ BEGIN {
   unless ($ENV{PERL_CORE}) {
     push @INC, $t_lib; # Let user's installed version override
   } else {
-    # We change directories, so expand @INC to absolute paths
+    # We change directories, so expand @INC and $^X to absolute paths
     # Also add .
     @INC = (map(File::Spec->rel2abs($_), @INC), ".");
+    $^X = File::Spec->rel2abs($^X);
 
     # we are in 't', go up a level so we don't create t/t/_tmp
     chdir '..' or die "Couldn't chdir to ..";
@@ -75,7 +77,7 @@ use Cwd ();
 
 # We pass everything through to Test::More
 use vars qw($VERSION @ISA @EXPORT %EXPORT_TAGS $TODO);
-$VERSION = 0.01;
+$VERSION = 0.01_01;
 @ISA = qw(Test::More); # Test::More isa Exporter
 @EXPORT = @Test::More::EXPORT;
 %EXPORT_TAGS = %Test::More::EXPORT_TAGS;
@@ -90,7 +92,7 @@ my @extra_exports = qw(
   find_in_path
   check_compiler
   have_module
-  ensure_blib
+  blib_load
 );
 push @EXPORT, @extra_exports;
 __PACKAGE__->export(scalar caller, @extra_exports);
@@ -99,18 +101,13 @@ __PACKAGE__->export(scalar caller, @extra_exports);
 
 ########################################################################
 
-{ # Setup a temp directory if it doesn't exist
+# always return to the current directory
+{ 
   my $cwd = Cwd::cwd;
-  my $tmp = File::Spec->catdir( $cwd, 't', '_tmp' . $$);
-  mkdir $tmp, 0777 unless -d $tmp;
 
-  sub tmpdir { $tmp }
   END {
-    if(-d $tmp) {
-      # Go back to where you came from!
-      chdir $cwd or die "Couldn't chdir to $cwd";
-      File::Path::rmtree($tmp) or diag "cannot clean dir '$tmp'";
-    }
+    # Go back to where you came from!
+    chdir $cwd or die "Couldn't chdir to $cwd";
   }
 }
 ########################################################################
@@ -124,6 +121,15 @@ __PACKAGE__->export(scalar caller, @extra_exports);
 }
 ########################################################################
 
+# Setup a temp directory 
+sub tmpdir { 
+  my ($self, $usr_tmp) = @_;
+  return File::Temp::tempdir( 'MB-XXXXXXXX', 
+    CLEANUP => 1, DIR => $ENV{PERL_CORE} ? Cwd::cwd : 
+                         $usr_tmp        ? $usr_tmp : File::Spec->tmpdir 
+  );
+}
+
 sub save_handle {
   my ($handle, $subr) = @_;
   my $outfile = temp_file_name();
@@ -190,28 +196,48 @@ sub check_compiler {
 
   local $SIG{__WARN__} = sub {};
 
+  blib_load('Module::Build');
   my $mb = Module::Build->current;
   $mb->verbose( 0 );
 
   my $have_c_compiler;
   stderr_of( sub {$have_c_compiler = $mb->have_c_compiler} );
 
-  return ($have_c_compiler, $mb->feature('C_support'));
+  # check noexec tmpdir
+  my $tmp_exec;
+  if ( $have_c_compiler ) {
+    my $dir = MBTest->tmpdir;
+    my $c_file = File::Spec->catfile($dir,'test.c');
+    open my $fh, ">", $c_file;
+    print {$fh} "int main() { return 0; }\n";
+    close $fh;
+    my $exe = $mb->cbuilder->link_executable(
+      objects => $mb->cbuilder->compile( source => $c_file )
+    );
+    $tmp_exec = 0 == system( $exe );
+  }
+  return ($have_c_compiler, $mb->feature('C_support'), $tmp_exec);
 }
 
 sub have_module {
   my $module = shift;
-  return eval "use $module; 1";
+  return eval "require $module; 1";
 }
 
-sub ensure_blib {
-  # Make sure the given module was loaded from blib/, not the larger system
+sub blib_load {
+  # Load the given module and ensure it came from blib/, not the larger system
   my $mod = shift;
+  have_module($mod) or die "Error loading $mod\: $@\n";
+
   (my $path = $mod) =~ s{::}{/}g;
-  
- SKIP: {
-    skip "no blib in core", 1 if $ENV{PERL_CORE};
-    like $INC{"$path.pm"}, qr/\bblib\b/, "Make sure $mod was loaded from blib/";
+  $path .= ".pm";
+  my ($pkg, $file, $line) = caller;
+  unless($ENV{PERL_CORE}) {
+    unless($INC{$path} =~ m/\bblib\b/) {
+      (my $load_from = $INC{$path}) =~ s{$path$}{};
+      die "$mod loaded from '$load_from'\nIt should have been loaded from blib.  \@INC contains:\n  ",
+      join("\n  ", @INC) . "\nFatal error occured in blib_load() at $file, line $line.\n";
+    }
   }
 }
 
